/*******************************************************************/
/*          VS1003 diriver for  Mega8 MP3 Player                   */
/*                                                                 */
/* Platform   : AVRStudio4.13 b528 + WinAVR20070122                */
/*              optimize -0s                                       */
/* Author     : bozai(Zhang Qibo)                                  */
/* E-mail     : sudazqb@163.com                                    */
/* MSN        : zhangqibo_1985@hotmail.com                         */
/* Date       : 2006-05-09                                         */
/*******************************************************************/
/*2007-11-22: add bass tremble control                             */
/*2007-10-21: add new functions,                                   */
/*2007-10-19: use new delay function                               */
/*2007-05-04: add slow start up code, and add enough delay         */
/*2007-04-21:                                                      */

#ifndef __VS1003B_H__
#define __VS1003B_H__

#define TREMBLE_VALUE	8		/* 0~15, 8 means off  */
#define TREMBLE_LOW_FS	6		/* 0~15, 0Hz-15KHz, lower frequency of tremble enhancement */
#define BASS_VALUE	4		/* 0~15, 0 means off  */
#define BASS_HIGH_FS	2		/* 2~15, up limit frequency of bass enhancement */

#define VS1003_AUDATA			0x05
#define VS1003_HEDATA0			0x08
#define VS1003_HEDATA1			0x09
#define VS1003_VOLUME	                0x0B

#define DEFAULT_BASS_TREMBLE ((TREMBLE_VALUE<<12)|(TREMBLE_LOW_FS<<8)|(BASS_VALUE<<4)|(BASS_HIGH_FS))

#define DEFAULT_VOLUME 0x3333	/* default volumn */

#define CLOCK_REG         0x47D0       //SCI CLOCKF = 0xFFD0  SC MULT = 2  SC ADD = 0  SC FREQ = 0x7D0 = 2000 XTALI = 2000*4000+8000000 = 16 MHz                                  
//    CLOCK_REG = 0x47D0 = 0b010 00 11111010000
//                           \\\ \\ \\\\\\\\\\\
//                           \\\ \\            ----> SC FREQ 00 ~ 10  Clock frequency                   2000 = 11111010000
//                           \\\   ----------------> SC ADD  11 ~ 12  Allowed multiplier addition       0    = 00
//                              -------------------> SC MULT 13 ~ 15  Clock multiplier                  2    = 010
flash unsigned int sample_rate_values[] = {0, 44100, 48000, 32000, 22050, 24000, 16000, 11025, 12000, 8000};

flash unsigned char sample_rate_values1[] = {11,12,8};
flash unsigned char sample_rate_values2[] = {22,24,16};
flash unsigned char sample_rate_values3[] = {44,48,32};
flash unsigned int  bit_rate_values1[]    = {0,0,40,48,56,64,80,96,112,128,160,192,224,256,320};
flash unsigned char bit_rate_values2[]    = {8,16,24,40,48,56,64,80,96,112,128,144,160};
flash char mode_values[4][13]             = {   "Stereo","JointStereo","DualChannel","Mono"};
typedef struct _AUDIO_DATA
        {
	unsigned int bitrate;
	unsigned int sample_rate;
	unsigned char is_stereo;
        }AUDIO_DATA;
        
typedef struct VS1003_Mp3Info
        {
	char sample_rate;
	char bit_rate; 
        char mode;
        }AUDIO_INFO;

#define VS1003B_XCS_DDR  DDRD
#define VS1003B_XCS_PORT PORTD
#define VS1003B_XCS_BIT  4

#define VS1003B_DREQ_DDR  DDRD
#define VS1003B_DREQ_PORT PORTD
#define VS1003B_DREQ_PIN  PIND
#define VS1003B_DREQ_BIT  6

#define VS1003B_XDCS_DDR  DDRD
#define VS1003B_XDCS_PORT PORTD
#define VS1003B_XDCS_BIT  7

#define VS1003B_XRESET_DDR  DDRD
#define VS1003B_XRESET_PORT PORTD
#define VS1003B_XRESET_BIT  5

#define VS1003B_SCK_DDR  DDRB
#define VS1003B_SCK_PORT PORTB
#define VS1003B_SCK_BIT  1

#define VS1003B_MOSI_DDR  DDRB
#define VS1003B_MOSI_PORT PORTB
#define VS1003B_MOSI_BIT  2

#define VS1003B_MISO_DDR  DDRB
#define VS1003B_MISO_PORT PORTB
#define VS1003B_MISO_BIT  3

/********************************************************************************/

/* port initialization */
#define PORT_INI(){\
        VS1003B_XCS_PORT |= 1<<VS1003B_XCS_BIT;\
        VS1003B_DREQ_PORT |= 1<<VS1003B_DREQ_BIT;\
        VS1003B_XRESET_PORT |= 1<<VS1003B_XRESET_BIT;\
        VS1003B_XDCS_PORT |= 1<<VS1003B_XDCS_BIT;\
        VS1003B_SCK_PORT |= 1<<VS1003B_SCK_BIT;\
        VS1003B_MOSI_PORT |= 1<<VS1003B_MOSI_BIT;\
        VS1003B_MISO_PORT |= 1<<VS1003B_MISO_BIT;\
        \
        VS1003B_XCS_DDR |= 1<<VS1003B_XCS_BIT;\
        VS1003B_DREQ_DDR &= ~(1<<VS1003B_DREQ_BIT);\
        VS1003B_XRESET_DDR |= 1<<VS1003B_XRESET_BIT;\
        VS1003B_XDCS_DDR |= 1<<VS1003B_XDCS_BIT;\
        VS1003B_SCK_DDR |= 1<<VS1003B_SCK_BIT;\
        VS1003B_MOSI_DDR |= 1<<VS1003B_MOSI_BIT;\
        VS1003B_MISO_DDR &= ~(1<<VS1003B_MISO_BIT);\
        }

#define VS1003B_XCS_H()         VS1003B_XCS_PORT |=  _BV(VS1003B_XCS_BIT)
#define VS1003B_XCS_L()         VS1003B_XCS_PORT &= ~_BV(VS1003B_XCS_BIT)
#define VS1003B_XRESET_H()      VS1003B_XRESET_PORT |=  _BV(VS1003B_XRESET_BIT)
#define VS1003B_XRESET_L()      VS1003B_XRESET_PORT &= ~_BV(VS1003B_XRESET_BIT)
#define VS1003B_XDCS_H()        VS1003B_XDCS_PORT |=  _BV(VS1003B_XDCS_BIT)
#define VS1003B_XDCS_L()        VS1003B_XDCS_PORT &= ~_BV(VS1003B_XDCS_BIT)

/*****************************************************************************************/

extern unsigned char VS1003B_Init(void);        /* Initialization, include prot and chip config */
extern void VS1003B_SoftReset(void);                /* Software reset, clear decode time and jump out midi or wma playing*/
extern unsigned int VS1003B_ReadDecodeTime(void);        /* Get current decode time, for time display */
extern void VS1003B_Fill2048Zero(void);        /* fill 32bytes zero, for wma and midi zero filling */
extern void VS1003B_Write32B(unsigned char * buf);        /* write 32bytes data */
extern unsigned char VS1003B_NeedData(void);        /* Check if it need data */
extern void VS1003B_SetVolume(unsigned int vol); /* Set volume */

/****************************************************************************************/
#endif
